#include "config.h"
#include "tft.h"
#include "lcm.h"
#include "timer.h"

#define TFT_SETREG16(reg, dat)      TFT_WriteReg(reg);   \
                                    TFT_WriteDataW(dat)

void TFT_WriteReg(BYTE reg)     
{
    LCM_WriteCmd_CS(reg);
}

void TFT_WriteData(BYTE dat)     
{
    LCM_WriteData_CS(dat);
}

void TFT_WriteDataW(WORD dat)     
{
    LCM_WriteData_CS((BYTE)(dat >> 8));
    LCM_WriteData_CS((BYTE)dat);
}

void TFT_Reset()
{
    LCM_CS = 1;
    delay_ms(50);    
    LCM_RST = 0;
    delay_ms(150);
    LCM_RST = 1;
    delay_ms(50);
}

void TFT_Init()
{
    TFT_Reset();            //ʼ֮ǰλ

#if (TFT_MODEL == ILI9325)

    TFT_SETREG16(0xE5, 0x78F0); 
    TFT_SETREG16(0x01, 0x0100); 
    TFT_SETREG16(0x02, 0x0700); 
    TFT_SETREG16(0x03, 0x1030); 
    TFT_SETREG16(0x04, 0x0000); 
    TFT_SETREG16(0x08, 0x0202);  
    TFT_SETREG16(0x09, 0x0000);
    TFT_SETREG16(0x0A, 0x0000); 
    TFT_SETREG16(0x0C, 0x0000); 
    TFT_SETREG16(0x0D, 0x0000);
    TFT_SETREG16(0x0F, 0x0000);
    TFT_SETREG16(0x10, 0x0000);   
    TFT_SETREG16(0x11, 0x0007);  
    TFT_SETREG16(0x12, 0x0000);  
    TFT_SETREG16(0x13, 0x0000); 
    TFT_SETREG16(0x07, 0x0000); 
    TFT_SETREG16(0x10, 0x1690);   
    TFT_SETREG16(0x11, 0x0227);
    delay_ms(10); 
    TFT_SETREG16(0x12, 0x009D);
    delay_ms(10);  
    TFT_SETREG16(0x13, 0x1900);
    delay_ms(10);  
    TFT_SETREG16(0x29, 0x0025);
    TFT_SETREG16(0x2B, 0x000D);
    TFT_SETREG16(0x30, 0x0007);
    TFT_SETREG16(0x31, 0x0303);
    TFT_SETREG16(0x32, 0x0003);
    TFT_SETREG16(0x35, 0x0206);
    TFT_SETREG16(0x36, 0x0008);
    TFT_SETREG16(0x37, 0x0406);
    TFT_SETREG16(0x38, 0x0304);
    TFT_SETREG16(0x39, 0x0007);
    TFT_SETREG16(0x3C, 0x0602);
    TFT_SETREG16(0x3D, 0x0008);
    TFT_SETREG16(0x50, 0x0000);
    TFT_SETREG16(0x51, 0x00EF);
    TFT_SETREG16(0x52, 0x0000);
    TFT_SETREG16(0x53, 0x013F);
    TFT_SETREG16(0x60, 0xA700);
    TFT_SETREG16(0x61, 0x0001);
    TFT_SETREG16(0x6A, 0x0000);
    TFT_SETREG16(0x80, 0x0000);
    TFT_SETREG16(0x81, 0x0000);
    TFT_SETREG16(0x82, 0x0000); 
    TFT_SETREG16(0x83, 0x0000); 
    TFT_SETREG16(0x84, 0x0000); 
    TFT_SETREG16(0x85, 0x0000); 
    TFT_SETREG16(0x90, 0x0010); 
    TFT_SETREG16(0x92, 0x0600); 
    TFT_SETREG16(0x07, 0x0133);
    TFT_SETREG16(0x00, 0x0022);

#ifdef HORIZONTAL
    TFT_SETREG16(0x03, 0x1038);
    TFT_SETREG16(0x01, 0x0000);
#else
    TFT_SETREG16(0x03, 0x1030);
    TFT_SETREG16(0x01, 0x0100);
#endif

#elif (TFT_MODEL == ILI9486)

    TFT_WriteReg(0xf2);
    TFT_WriteData(0x18);
    TFT_WriteData(0xa3);
    TFT_WriteData(0x12);
    TFT_WriteData(0x02);
    TFT_WriteData(0xb2);
    TFT_WriteData(0x12);
    TFT_WriteData(0xff);
    TFT_WriteData(0x10);
    TFT_WriteData(0x00);
    TFT_WriteReg(0xf8);
    TFT_WriteData(0x21);
    TFT_WriteData(0x04);
    TFT_WriteReg(0xf9);
    TFT_WriteData(0x00);
    TFT_WriteData(0x08);
    TFT_WriteReg(0x36);
    TFT_WriteData(0x08);
    TFT_WriteReg(0x3a);
    TFT_WriteData(0x05);
    TFT_WriteReg(0xb4);
    TFT_WriteData(0x01);
    TFT_WriteReg(0xb6);
    TFT_WriteData(0x02);
    TFT_WriteData(0x22);
    TFT_WriteReg(0xc1);
    TFT_WriteData(0x41);
    TFT_WriteReg(0xc5);
    TFT_WriteData(0x00);
    TFT_WriteData(0x07);
    TFT_WriteReg(0xe0);
    TFT_WriteData(0x0f);
    TFT_WriteData(0x1f);
    TFT_WriteData(0x1c);
    TFT_WriteData(0x0c);
    TFT_WriteData(0x0f);
    TFT_WriteData(0x08);
    TFT_WriteData(0x48);
    TFT_WriteData(0x98);
    TFT_WriteData(0x37);
    TFT_WriteData(0x0a);
    TFT_WriteData(0x13);
    TFT_WriteData(0x04);
    TFT_WriteData(0x11);
    TFT_WriteData(0x0d);
    TFT_WriteData(0x00);
    TFT_WriteReg(0xe1);
    TFT_WriteData(0x0f);
    TFT_WriteData(0x32);
    TFT_WriteData(0x2e);
    TFT_WriteData(0x0b);
    TFT_WriteData(0x0d);
    TFT_WriteData(0x05);
    TFT_WriteData(0x47);
    TFT_WriteData(0x75);
    TFT_WriteData(0x37);
    TFT_WriteData(0x06);
    TFT_WriteData(0x10);
    TFT_WriteData(0x03);
    TFT_WriteData(0x24);
    TFT_WriteData(0x20);
    TFT_WriteData(0x00);
    TFT_WriteReg(0x11);
    delay_ms(120);
    TFT_WriteReg(0x29);

    TFT_WriteReg(0x36);
#ifdef HORIZONTAL
    TFT_WriteData(0x68);
#else
    TFT_WriteData(0x08);
#endif

#endif
}

void TFT_ShowStart()
{    
#if (TFT_MODEL == ILI9325)

    TFT_SETREG16(0x50, 0);
    TFT_SETREG16(0x51, TFT_W - 1);
    TFT_SETREG16(0x52, 0);
    TFT_SETREG16(0x53, TFT_H - 1);
    TFT_SETREG16(0x20, 0);
    TFT_SETREG16(0x21, 0);
    
    TFT_WriteReg(0x22);
    
#elif (TFT_MODEL == ILI9486)

    TFT_WriteReg(0x2a);
    TFT_WriteDataW(0);
    TFT_WriteDataW(TFT_W - 1);
    TFT_WriteReg(0x2b);
    TFT_WriteDataW(0);
    TFT_WriteDataW(TFT_H - 1);
    
    TFT_WriteReg(0x2c);
    
#endif
    LCM_CS = 0;
}

void TFT_ShowData(BYTE dat)
{
    LCM_WriteData(dat);
}

void TFT_ShowDataW(WORD dat)
{
    LCM_WriteData((BYTE)(dat >> 8));
    LCM_WriteData((BYTE)dat);
}

void TFT_ShowEnd()
{
    LCM_CS = 1;
}

